// ==================================================================
//                                 NOTICE
//                        COPYRIGHT© 2017 Revulytics, Inc.
//                            ALL RIGHTS RESERVED
//
//      This program is confidential, proprietary, and a trade
//   secret of Revulytics, Inc. The receipt or possession of
//     this program does not convey any rights to reproduce or
//      disclose its contents, or to manufacture, use, or sell
//       anything that it may describe, in whole or in part,
//   without the specific written consent of Revulytics, Inc.
// ==================================================================
//
// SDK Version: 4.0.7 Linux
// Release date: May 2017
//
// You may get the latest version from: http://devzone.revulytics.com
// For customer support please visit: http://helpdesk.revulytics.com
// ==================================================================

#ifndef TRACKERBIRD_H
#define TRACKERBIRD_H

#ifdef __cplusplus
extern "C" {
#else
#include <stdbool.h>
#endif

#define TB_OK                       1  //Normal execution.  NOTE:  TB_START_OK (see function tbStartStatus) has different value.
#define TB_FUNCTION_NOT_AVAIL      -1  //Function not enabled, usage level exceeded, or timeout with the Trackerbird Server.
#define TB_CONN_ERROR              -2  //General problem connecting with the Trackerbird Server.  See function tbConnectionCheck.
#define TB_AUTH_FAILURE            -3  //Login denied by the Trackerbird Server.
#define TB_SERVER_ERROR            -4  //General problem with the Trackerbird Server.
#define TB_CONFIG_NOT_CREATED      -5  //Function not allowed due to Trackerbird SDK state is prior to successful tbCreateConfig
                                       //that loads Configuration File.
#define TB_CONFIG_ALREADY_CREATED  -6  //Function not allowed due to Trackerbird SDK state is after successful tbCreateConfig.
#define TB_NOT_STARTED             -7  //Function not allowed due to Trackerbird SDK state is prior to successful tbStart or
                                       //after unsuccessful tbStart.
#define TB_INVALID_PARAMETER       -9  //Unexpected function parameter value.
#define TB_INVALID_PATH           -10  //FilePath for Trackerbird Configuration is ill-formed or does not exist.
#define TB_ACCESS_DENIED          -11  //Permission issues with the Configuration FilePath.
#define TB_FAILED_DEPENDENCY      -12  //[LINUX only]Problem locating or loading a required dynamic library.
#define TB_FILE_PATH_NOT_SET      -13  //Function not allowed due to Trackerbird SDK state is prior to tbSetFilePath.
#define TB_APP_STOPPED            -14  //Function not allowed due to Trackerbird SDK state is after tbStop.
#define TB_INTERNAL_ERROR         -99  //General unrecoverable error.  NOTE:  Once TB_INTERNAL_ERROR is returned from any
                                       //Trackerbird SDK call, the Client should not make any more Trackerbird SDK calls.
                                       //Any subsequent calls will also return TB_INTERNAL_ERROR.

#define TB_START_OK                 2  //[tbStartStatus only]Normal execution.  NOTE:  TB_OK has different value.
#define TB_START_LOADING            1  //[tbStartStatus only]Non-error return; Configuration is still loading.
#define TB_START_NOT_CALLED         0  //[tbStartStatus only]Function not allowed due to Trackerbird SDK state is prior to
                                       //successful tbStart.
#define TB_START_CONNECTION_ERROR  -2  //[tbStartStatus only]General problem connecting with the Trackerbird Server.
#define TB_START_AUTH_FAILURE      -3  //[tbStartStatus only]Login denied by the Trackerbird Server.
#define TB_START_SERVER_ERROR      -4  //[tbStartStatus only]General problem with the Trackerbird Server.  

#define TB_CONNCHECK_MISMATCH      -1  //URL and Product ID are mismatched for connection with the Trackerbird Server.

#define TB_KEYTYPE_UNCHANGED       -1  //Parameter value - Key type is Unchanged (when in param); not used as out param.
#define TB_KEYTYPE_EVALUATION       0  //Parameter value - Key is of type Evaluation.
#define TB_KEYTYPE_PURCHASED        1  //Parameter value - Key is of type Purchased.
#define TB_KEYTYPE_FREEWARE         2  //Parameter value - Key is of type Freeware.
#define TB_KEYTYPE_UNKNOWN          3  //Parameter value - Key is of type Unknown.
#define TB_KEYTYPE_NFR              4  //Parameter value - Key is of type Not For Resale.
#define TB_KEYTYPE_CUSTOM1          5  //Parameter value - Key is of type User Defined - Custom1.
#define TB_KEYTYPE_CUSTOM2          6  //Parameter value - Key is of type User Defined - Custom2.
#define TB_KEYTYPE_CUSTOM3          7  //Parameter value - Key is of type User Defined - Custom3.

#define TB_KEY_STATUS_UNCHANGED    -1  //Parameter value - Key Status Flag is Unchanged (when in param) or Not Known (when out param).
#define TB_KEY_STATUS_NO            0  //Parameter value - Key Status Flag is No.
#define TB_KEY_STATUS_YES           1  //Parameter value - Key Status Flag is Yes.

                                       //NOTE:  The Index order below is different than the parameter order in function tbSetLicense.
#define TB_KEY_ACTIVE_INDEX         0  //Index for License Array - Status Flag for Key being Activated.
#define TB_KEY_BLACKLIST_INDEX      1  //Index for License Array - Status Flag for Key being Blacklisted.
#define TB_KEY_EXPIRED_INDEX        2  //Index for License Array - Status Flag for Key being Expired.
#define TB_KEY_WHITELIST_INDEX      3  //Index for License Array - Status Flag for Key being Whitelisted.
#define TB_KEY_TYPE_INDEX           4  //Index for License Array - Key Type value.

#define TB_MESSAGETYPE_ALL          0  //Parameter value - ReachOut™ message type any.
#define TB_MESSAGETYPE_TEXT         1  //Parameter value - ReachOut™ message type Text only.
#define TB_MESSAGETYPE_URL          2  //Parameter value - ReachOut™ message type URL only.

#define TB_START_SYNC               0  //[tbStart only]Parameter value - Perform Sync and ReachOut™ during tbStart.
#define TB_START_NO_REACHOUT       -1  //[tbStart only]Parameter value - Perform Sync but no ReachOut™ during tbStart.
#define TB_START_NO_SYNC           -2  //[tbStart only]Parameter value - Perform no Sync nor ReachOut™ during tbStart.

#define TB_STOP_SYNC                0  //[tbStop only]Parameter value - Perform Sync during tbStop.
#define TB_STOP_NO_SYNC            -1  //[tbStop only]Parameter value - Perform no Sync during tbStop.

#ifdef __cplusplus
#define TBINSTANCE TbInstance
#else
#define TBINSTANCE struct TbInstance
#endif

typedef int TBRESULT;
struct TbInstance;

/**
 * @details This function is the required first call to create a Trackerbird SDK Instance.  The pointer returned is used in all
 * subsequent Trackerbird SDK API calls.  Function tbCreateInstance allocates the internal resources for the Trackerbird SDK,
 * including memory, threads, and locks, and on the Linux platform, loads dependent libraries dynamically.  Function
 * tbCreateInstance does not create, initialize, nor start the Trackerbird SDK Configuration.  See functions tbSetFilePath
 * (@see tbSetFilePath), tbCreateConfig (@see tbCreateConfig) and tbStart (@see tbStart) for more information on initializing
 * a Trackerbird SDK Configuration.
 *
 * The last call to a Trackerbird SDK Instance is to destroy the Instance (@see tbDestroyInstance).
 *
 * IMPORTANT - Do not call function tbFree (@see tbFree) with the pointer returned from function tbCreateInstance.
 *
 * @exception None.
 *
 * @return One of the following values:
 * * NULL     - Unrecoverable error during Trackerbird SDK creation.
 * * Non-NULL - The pointer to the newly allocated Trackerbird SDK Instance.
 */
TBINSTANCE* tbCreateInstance();
    
/**
 * @details This function is the required last call to a Trackerbird SDK Instance.  Function tbDestroyInstance frees up all
 * Trackerbird SDK internal resources, including memory, threads, and locks, and on the Linux platform, closes dependent
 * libraries that were loaded dynamically.  Prior to calling function tbDestroyInstance, the Trackerbird SDK should have
 * been stopped (@see tbStop).  The Trackerbird SDK Instance pointer can no longer be used for any Trackerbird SDK API calls
 * after the call to function tbDestroyInstance.
 *
 * IMPORTANT - Do not call function tbFree (@see tbFree) with the pointer to the Trackerbird SDK Instance.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                (  1) - Normal execution.
 * * TB_INVALID_PARAMETER ( -9) - tbInstance is NULL.
 * * TB_INTERNAL_ERROR    (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbDestroyInstance(TBINSTANCE* tbInstance);
    
/**
 * @details This function returns the SDK Version String for this Trackerbird SDK Instance.  The SDK Version String contains
 * the following formatted information:
 *   Trackerbird SDK version [VER_NUM] for [PLATFORM]/[SHIM]
 *   Where
 *   * VER_NUM      is X.Y.Z (currently 4.0.7)
 *   * PLATFORM     is WINDOWS, OSX or LINUX
 *   * SHIM         is only provided if using a shim layer and if provided will be ObjC, C or .NET
 *
 * IMPORTANT - Do not call function tbFree (@see tbFree) with the pointer to the Trackerbird SDK Version String.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @exception None.
 *
 * @return One of the following values:
 * * NULL     - tbInstance is NULL, or the Trackerbird SDK Instance is in a TB_INTERNAL_ERROR state.
 * * Non-NULL - The pointer to the SDK Version String.
 */
const char* tbGetSDKVersion(TBINSTANCE* tbInstance);
    
/**
 * @details This function is used to set the minimum required information to start the Trackerbird SDK Instance.  This function
 * should only be called after function tbCreateInstance (@see tbCreateInstance).  Function tbSetFilePath (@see tbSetFilePath)
 * must be called before function tbCreateConfig to establish the directory for the Trackerbird SDK Data Files, including the
 * Configuration File.  Function tbCreateConfig should only be called once for an SDK Instance.
 *
 * IMPORTANT - The first time function tbCreateConfig is called, a Configuration File (tbconfig.xml) is created in the
 * designated Data File Path (@see tbSetFilePath).  This Configuration File endures beyond the lifetime of the execution of the
 * Trackerbird SDK Instance, and is consulted on subsequent runs of Trackerbird SDK.  The Configuration File contains most SDK
 * Configuration values including the CallHome URL (@see url).  Once this Configuration File is created, and detected by
 * function tbCreateConfig, the CallHome URL parameter (@see url) supplied to function tbCreateConfig is ignored.
 *
 * IMPORTANT - The Trackerbird Configuration (and hence the Trackerbird SDK Configuration File) may even be shared between
 * multiple SDK Instances, if those SDK Instances use the same Trackerbird SDK Data File Path (@see tbSetFilePath) and same
 * Product ID (@see productID).
 *
 * IMPORTANT - The CallHome URL (@see url) and Product ID (@see productID) are inter-related.  Every Trackerbird Product
 * has a unique URL. Therefore, if during testing one Product ID is used, and then a different Product ID is registered for
 * production, it is important to delete all Trackerbird SDK Data Files.  Otherwise, the Trackerbird SDK Clients will continue
 * to use the testing URL that will now mismatch with the production Product ID.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param url
 * CallHome URL: Every product registered with Trackerbird has its own unique CallHome URL usually in the form
 * ‘http://xxxxx.tbnet1.com’. This URL is generated automatically on Account Creation and is used by the Trackerbird SDK to
 * communicate with the Trackerbird server. This URL can be obtained from the Customer Area / Developer Zone.
 *
 * IMPORTANT - url is ignored if a Configuration File already exists, and the URL value within the Configuration File is used
 * instead. This is done so that the Trackerbird Server can instruct the Trackerbird SDK to use a new URL in cases of change.
 * 
 * @param productID
 * This is a unique 10-digit Product ID number that identifies a specific product with the Trackerbird Server. This Product ID
 * is obtained from the Customer Area / Developer Zone.  The combination of Data File Path (@see setFilePath) and Product ID
 * determines whether or not a Configuration File is shared between multiple Trackerbird SDK Instances.
 *
 * @param productVersion
 * The Version Number of the Trackerbird SDK Client (application or plugin) being run.
 *
 * @param productBuildNumber
 * The Build Number of the Trackerbird SDK Client (application or plugin) being run.
 *
 * @param multiSessionEnabled
 * Whether multiple user sessions can be present in a single Trackerbird SDK Instance.  This would typically be false for single
 * user applications, and true for multi-user (e.g., server) applications.  When multiSessionEnabled is false, functions
 * tbSessionStart (@see tbSessionStart) and tbSessionStop (@see tbSessionStop) are disallowed, and the single session is implied
 * by functions tbStart (@see tbStart) and tbStop (@see tbStop).  When multiSessionEnabled is true, functions tbStart (@see
 * tbStart) and tbStop (@see tbStop) are still required but do not imply a session (i.e., do not record anything.)  When
 * multiSessionEnabled is true, functions tbSessionStart (@see tbSessionStart) and tbSessionStop (@see tbSessionStop) are
 * required to delimit sessions and track events.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                     (  1) - Normal execution.
 * * TB_CONFIG_ALREADY_CREATED ( -6) - Function not available; tbCreateConfig has been successfully called before.
 * * TB_INVALID_PARAMETER      ( -9) - tbInstance is NULL, URL is NULL or empty, or productID is NULL or empty.
 * * TB_FAILED_DEPENDENCY      (-12) - [LINUX only] External dynamic library not located or not successfully loaded.
 * * TB_FILE_PATH_NOT_SET      (-13) - A successful call to tbSetFilePath (@see tbSetFilePath) was not made before this call.
 * * TB_INTERNAL_ERROR         (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbCreateConfig(TBINSTANCE* tbInstance, const char* url, const char* productID, const char* productVersion,
                        const char* productBuildNumber, bool multiSessionEnabled);

/**
 * @details This function must be called before utilizing any of the Tracking or Synchronization functions of the Trackerbird
 * SDK.  Before calling function tbStart, the SDK configuration must be properly set (@see tbCreateConfig).  Function tbStart
 * should only be called once, and should be paired with a call to tbStop (@see tbStop).  Depending on the value of
 * multiSessionEnabled (@see tbCreateConfig), the function tbStart may start an implied Session.
 *
 * IMPORTANT - Function tbStart can only be called successfully once.  Calling function tbStart again even after calling
 * function tbStop, will return TB_FUNCTION_NOT_AVAILABLE.
 *
 * IMPORTANT - Function tbStart has different return values than function tbStartStatus (@see tbStartStatus).
 *
 * IMPORTANT - Synchronization involves sending cached data stored during this (or previous) Trackerbird SDK execution from the
 * Trackerbird SDK Client to the Trackerbird Server, and receiving Configuration updates for this Trackerbird SDK Instance from
 * the Trackerbird Server, including, potentially, a ReachOut™ message.  The Synchronization process runs in the background
 * during Trackerbird SDK execution.  The Synchronization process is triggered periodically automatically (@see tbStartAutoSync
 * and @see tbStopAutoSync), and on-demand (@see tbStart, @see tbStop, and @see tbSync).
 * 
 * For more information on ReachOut™ messages, @see tbMessageCheck and @see tbMessageCheckWithMessageType.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param doSync
 * Determines if the Trackerbird SDK will Synchronize with the Trackerbird Server immediately at start up.  Normally, doSync is
 * set to TB_START_SYNC (0).  The possible values are as follows:
 * * TB_START_SYNC        ( 0) - Start on-demand Synchronization and get ReachOut™ message
 * * TB_START_NO_REACHOUT (-1) - Start on-demand Synchronization but do not get ReachOut™ message
 * * TB_START_NO_SYNC     (-2) - Do not start on-demand Synchronization
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_FUNCTION_NOT_AVAIL ( -1) - Function ignored; function tbStart has already been successfully called.
 * * TB_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL, or doSync is not a legal value.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbStart(TBINSTANCE* tbInstance, int doSync);

/**
 * @details This function must be called after function tbStart (@see tbStart), and should always be called when the Trackerbird
 * SDK client is exiting.  Calling function tbStop helps the Trackerbird Server to accurately calculate session duration, which
 * is reported in the application usage statistics. Calling function tbStop indicates that the SDK Client was terminated
 * gracefully without any unhandled exceptions.  Unlike most Trackerbird SDK functions, function tbStop is synchronous, meaning
 * it returns only once finished.  Hence, it is recommended to close/hide any application UI before calling function tbStop.
 *
 * IMPORTANT - Synchronization involves sending cached data stored during this (or previous) Trackerbird SDK execution from the
 * Trackerbird SDK Client to the Trackerbird Server, and receiving Configuration updates for this Trackerbird SDK Instance from
 * the Trackerbird Server, including, potentially, a ReachOut™ message.  The Synchronization process runs in the background
 * during Trackerbird SDK execution.  The Synchronization process is triggered periodically automatically (@see tbStartAutoSync
 * and @see tbStopAutoSync), and on-demand (@see tbStart, @see tbStop, and @see tbSync).
 * 
 * Function tbStop does not request ReachOut™ messages during Synchronization.
 *
 * IMPORTANT - Performing a Synchronization will take time.  If the Trackerbird SDK Client needs to exit immediately, setting
 * doSync to TB_STOP_NO_SYNC (-1) will be the fastest exit.  However, this will defer Synchronization until the next execution
 * of this Trackerbird SDK instance (@see tbStart, @see tbSync).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param doSync
 * Determines if the Trackerbird SDK will Synchronize with the Trackerbird Server as part of the SDK stop.  Normally, doSync is
 * set to TB_STOP_SYNC (0).  The possible values are as follows:
 * * TB_STOP_SYNC      ( 0) - Start on-demand Synchronization and allow it to finish regardless of amount of time taken
 * * TB_STOP_NO_SYNC   (-1) - Do not start on-demand Synchronization
 * * Any numeric value >= 1 - Start on-demand Synchronization and allow it to run for a maximum of 'doSync' seconds.  If the
 *       Synchronization is not completed, it is interrupted/aborted, and the data is preserved for a subsequent execution of
 *       Trackerbird SDK.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL, or doSync is not a legal value.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbStop(TBINSTANCE* tbInstance, int doSync);

/**
 * @details This function will start an on-demand Synchronization.  In general, the periodic automatic Synchronization is
 * sufficient, and there is no need to call function tbSync.  Function tbSync returns immediately, with the Synchronization
 * process running asynchronously in the background.  Hence, there is no need to place a call to function tbSync inside a
 * separate Client thread.
 *
 * IMPORTANT - Synchronization involves sending cached data stored during this (or previous) Trackerbird SDK execution from the
 * Trackerbird SDK Client to the Trackerbird Server, and receiving Configuration updates for this Trackerbird SDK Instance from
 * the Trackerbird Server, including, potentially, a ReachOut™ message.  The Synchronization process runs in the background
 * during Trackerbird SDK execution.  The Synchronization process is triggered periodically automatically (@see tbStartAutoSync
 * and @see tbStopAutoSync), and on-demand (@see tbStart, @see tbStop, and @see tbSync).
 *
 * For more information on ReachOut™ messages, @see tbMessageCheck and @see tbMessageCheckWithMessageType.
 *
 * Both the Trackerbird SDK and the Trackerbird Server can reject an on-demand Synchronization request from occurring.  This is
 * done to prevent abuse and unnecessary server load.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param getReachout
 * Determines if the Trackerbird Server will send a ReachOut™ message during this particular Synchronization, if available.
 * Normally, the value should be true.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - Function not available due to empty Client ID.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSync(TBINSTANCE* tbInstance, bool getReachout);

/**
 * @details This function will start the automatic periodic Synchronization in the background.  This is usually only required
 * for Clients that cannot call function tbStop for Synchronization, such as background services or web services whose running
 * cycle (session runtime duration) spans over 6 hours.  Before using this function please make sure you read the section on
 * Caching and Synchronizing in the on-line documentation.  The automatic periodic Synchronization can be stopped with function
 * tbStopAutoSync (@see tbStopAutoSync).  After stopping the automatic Synchronization, it can be re-started with another call
 * to function tbStartAutoSync.
 *
 * IMPORTANT - Synchronization involves sending cached data stored during this (or previous) Trackerbird SDK execution from the
 * Trackerbird SDK Client to the Trackerbird Server, and receiving Configuration updates for this Trackerbird SDK Instance from
 * the Trackerbird Server, including, potentially, a ReachOut™ message.  The Synchronization process runs in the background
 * during Trackerbird SDK execution.  The Synchronization process is triggered periodically automatically (@see tbStartAutoSync
 * and @see tbStopAutoSync), and on-demand (@see tbStart, @see tbStop, and @see tbSync).
 *
 * For more information on ReachOut™ messages, @see tbMessageCheck and @see tbMessageCheckWithMessageType.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param getReachout
 * Determines if the Trackerbird Server will send a ReachOut™ message during automatic Synchronization, if available.  Normally,
 * the value should be true.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbStartAutoSync(TBINSTANCE* tbInstance, bool getReachout);

/**
 * @details This function will stop the automatic periodic Synchronization in the background.  For more information see function
 * tbStartAutoSync (@see tbStartAutoSync).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbStopAutoSync(TBINSTANCE* tbInstance);

/**
 * @details This function starts a new User Session with the supplied unique Session ID.  Functions tbSessionStart and
 * tbSessionStop (@see tbSessionStop) should be called only if multiSessionEnabled was set to true in function tbCreateConfig
 * (@see tbCreateConfig).  Function tbSessionStart records the time of the call to use in the Trackerbird Session Duration
 * calculation.  More than one User Session can exist simultaneously, each with an unique Session ID.  Trackerbird Event
 * Tracking can be associated to a given User Session (@see tbEventTrack, @see tbEventTrackNum, and @see tbEventTrackTxt).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param sessionID
 * An unique ID that refers to the user session that is being started.  This same Session ID should later be used for Event
 * Tracking.  The value must not be NULL, must not be empty, and must be unique.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - tbCreateConfig was not called with multiSessionEnabled = true.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL, or sessionID is NULL, empty, or not unique (in use).
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSessionStart(TBINSTANCE* tbInstance, const char* sessionID);

/**
 * @details This function ends the User Session for the supplied Session ID.  Functions tbSessionStart (@see tbSessionStart) and
 * tbSessionStop should be called only if multiSessionEnabled was set to true in function tbCreateConfig (@see tbCreateConfig).
 * Function tbSessionStop records the time of the call to use in the Trackerbird Session Duration calculation.  If function
 * tbSessionStop is never called for a given Session ID, eventually the Trackerbird SDK will consider the User Session as "timed
 * out", and the last recorded Event (@see tbEventTrack, @see tbEventTrackNum, and @see tbEventTrackTxt) for the User Session
 * will be used as the User Session Stop Time.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param sessionID
 * A Session ID previously used in a call to function tbSessionStart (@see tbSessionStart) that has not yet been used in a call
 * to function tbSessionStop.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - tbCreateConfig was not called with multiSessionEnabled = true.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL, or sessionID not recognized (NULL, empty, not used in tbStart,
 *                                    already used in tbStop).
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSessionStop(TBINSTANCE* tbInstance, const char* sessionID);

/**
 * @details This function sets the Trackerbird SDK Data File Path that will be used to locate, create and save its data files.
 * Function tbSetFilePath must be called before calling function tbCreateConfig (@see tbCreateConfig), and cannot be called
 * after calling function tbCreateConfig.
 *
 * IMPORTANT - The combination of Trackerbird SDK Data File Path and Product ID (@see tbCreateConfig) determine whether or not
 * the Trackerbird Configuration is shared between multiple Trackerbird SDK Instances.
 *
 * IMPORTANT - The Trackerbird SDK Client application must have read/write access to the Data File Path.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param filePath
 * The location to use for the Trackerbird SDK Data Files.  The value must not be NULL nor empty.  The value must refer to a
 * directory that exists and has write permissions for this Trackerbird Client process.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                     (  1) - Normal execution.
 * * TB_CONFIG_ALREADY_CREATED ( -6) - Function ignored as filePath cannot be changed after call to tbCreateConfig.
 * * TB_INVALID_PARAMETER      ( -9) - tbInstance is NULL, or filePath is NULL or empty.
 * * TB_INVALID_PATH           (-10) - filePath does not exist.
 * * TB_ACCESS_DENIED          (-11) - filePath does not have write permissions.
 * * TB_INTERNAL_ERROR         (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSetFilePath(TBINSTANCE* tbInstance, const char* filePath);

/**
 * @details This function sets the Product Edition to the supplied value.  An example of this would be to distinguish between
 * "Home" and "Business" modes within a single Product ID (@see tbCreateConfig).  Function tbSetProductEdition does not have
 * to be called, but if it is called, it must be after calling function tbCreateConfig (@see tbCreateConfig).  Function
 * tbSetProductEdition can be called more than once.
 *
 * IMPORTANT - Function tbSetProductEdition does not have to be called, but if it is called, it must be after calling function
 * tbCreateConfig (@see tbCreateConfig), and ideally before function tbStart (@see tbStart), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function tbStart, or on the next sync if set after calling function tbStart.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param productEdition
 * The Product Edition value to use.  A NULL or empty value removes any previous value.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSetProductEdition(TBINSTANCE* tbInstance, const char* productEdition);

/**
 * @details This function sets the Product Language to the supplied value.  An example of this would be to distinguish the
 * language a user views an internationalized product in.  Note, Product Language is different than the OS Language that is
 * automatically collected by the Trackerbird SDK.
 *
 * IMPORTANT - Function tbSetProductLanguage does not have to be called, but if it is called, it must be after calling function
 * tbCreateConfig (@see tbCreateConfig), and ideally before function tbStart (@see tbStart), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function tbStart, or on the next sync if set after calling function tbStart.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param productLanguage
 * The Product Language value to use.  A NULL or empty value removes any previous value.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSetProductLanguage(TBINSTANCE* tbInstance, const char* productLanguage);

/**
 * @details This function sets the Product Version to the supplied value.  In general, function tbSetProductVersion does not
 * need to be called, as the Product Version is set during tbCreateConfig (@see tbCreateConfig).  Function tbSetProductVersion
 * is used to change the Product Version from its original value, for example, in the case of an on-line Client update.
 *
 * IMPORTANT - Function tbSetProductVersion does not have to be called, but if it is called, it must be after calling function
 * tbCreateConfig (@see tbCreateConfig), and ideally before function tbStart (@see tbStart), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function tbStart, or on the next sync if set after calling function tbStart.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param productVersion
 * The Product Version value to use.  A NULL or empty value removes any previous value.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSetProductVersion(TBINSTANCE* tbInstance, const char* productVersion);

/**
 * @details This function sets the Product Build Number to the supplied value.  In general, function tbSetProductBuildNumber
 * does not need to be called, as the Product Build Number is set during tbCreateConfig (@see tbCreateConfig).  Function
 * tbSetProductBuildNumber is used to change the Product Build Number from its original value, for example, in the case of an
 * on-line Client update.
 *
 * IMPORTANT - Function tbSetProductBuildNumber does not have to be called, but if it is called, it must be after calling function
 * tbCreateConfig (@see tbCreateConfig), and ideally before function tbStart (@see tbStart), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function tbStart, or on the next sync if set after calling function tbStart.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param productBuildNumber
 * The Product Build Number value to use.  A NULL or empty value removes any previous value.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSetProductBuildNumber(TBINSTANCE* tbInstance, const char* productBuildNumber);

/**
 * @details This function sets the License Information being used by the Trackerbird SDK Client.  The License Information
 * includes a License Key Type and four (4) License Status Flags -- keyExpired, keyActivated, keyBlacklisted, and
 * keyWhitelisted.  The License Key Type can have one of eight (8) values listed below, including any of three (3) custom values
 * that may be used freely by the Client.  If a License Status Flag value is not to be changed, the value
 * TB_KEY_STATUS_UNCHANGED (-1) can be used for the corresponding parameter.  The License Key Type can also be indicated as
 * unchanged with value TB_KEYTYPE_UNCHANGED (-1).
 *
 * IMPORTANT - Function tbSetLicense does not have to be called, but if it is called, it must be after calling function
 * tbCreateConfig (@see tbCreateConfig), and ideally before function tbStart (@see tbStart), especially if this is a new
 * installation.  For a new installation, the value set will be communicated to the server immediately if set prior to calling
 * function tbStart, or on the next sync if set after calling function tbStart.
 *
 * IMPORTANT - For a new installation, not calling function tbSetLicense before function tbStart (@see tbStart) may result in
 * anomolies in the License Funnel Report, as the Key Type will initially be recorded as "Unknown" and later recorded as the set
 * value.
 *
 * IMPORTANT - The parameter order in function tbSetLicense is not the same as the index order in the License Array (see Index
 * #defines TB_KEY_[NAME]_INDEX.)
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param keyType
 * The Key Type to set.  The possible values are as follows:
 * * TB_KEYTYPE_UNCHANGED  (-1)
 * * TB_KEYTYPE_EVALUATION ( 0)
 * * TB_KEYTYPE_PURCHASED  ( 1)
 * * TB_KEYTYPE_FREEWARE   ( 2)
 * * TB_KEYTYPE_UNKNOWN    ( 3)
 * * TB_KEYTYPE_NFR        ( 4)
 * * TB_KEYTYPE_CUSTOM1    ( 5)
 * * TB_KEYTYPE_CUSTOM2    ( 6)
 * * TB_KEYTYPE_CUSTOM3    ( 7)
 *
 * @param keyExpired
 * The License Status Flag to indicate whether the Client license has expired.  The possible values are as follows:
 * * TB_KEY_STATUS_UNCHANGED (-1)
 * * TB_KEY_STATUS_NO        ( 0)
 * * TB_KEY_STATUS_YES       ( 1)
 *
 * @param keyActivated
 * The License Status Flag to indicate whether the Client license been activated.  The possible values are as follows:
 * * TB_KEY_STATUS_UNCHANGED (-1)
 * * TB_KEY_STATUS_NO        ( 0)
 * * TB_KEY_STATUS_YES       ( 1)
 *
 * @param keyBlacklisted
 * The License Status Flag to indicate whether the Client license has been blacklisted.  The possible values are as follows:
 * * TB_KEY_STATUS_UNCHANGED (-1)
 * * TB_KEY_STATUS_NO        ( 0)
 * * TB_KEY_STATUS_YES       ( 1)
 *
 * @param keyWhitelisted
 * The License Status Flag to indicate whether the Client license has been whitelisted.  The possible values are as follows:
 * * TB_KEY_STATUS_UNCHANGED (-1)
 * * TB_KEY_STATUS_NO        ( 0)
 * * TB_KEY_STATUS_YES       ( 1)
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL; or keyType, keyExpired, keyActivated, keyBlacklisted or keyWhitelisted
 *                                 is not a legal value.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSetLicense(TBINSTANCE* tbInstance, int keyType, int keyExpired, int keyActivated, int keyBlacklisted,
                      int keyWhitelisted);

/**
 * @details This function sets one of the twenty (20) Custom Properties to the supplied value.  On the Trackerbird Web UI,
 * Custom Values are given an ID ranging from C01 to C20, corresponding to the supplied index to function tbSetCustomProperty.
 *
 * IMPORTANT - Function tbSetCustomProperty does not have to be called, but if it is called, it must be after calling function
 * tbCreateConfig (@see tbCreateConfig).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param index
 * A value between 1 and 20.  NOTE, this is not a zero-based index.
 *
 * @param customValue
 * The value to be assigned to the indexed Custom Property.  A NULL or empty value removes any previous value. The maximum
 * length of customValue is 1024 characters not including the terminating NULL.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL; or index out of range (1-20); or customValue max length exceeded.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbSetCustomProperty(TBINSTANCE* tbInstance, unsigned short index, const char* customValue);

/**
 * @details This function returns the current status of function tbStart (@see tbStart).  NOTE, function tbStart (@see tbStart)
 * executes asynchronously, so the status returned by tbStartStatus may change over time.
 *
 * IMPORTANT - Function tbStartStatus does not have to be called, but if it is called, it must be after calling function
 * tbStart (@see tbStart).
 *
 * IMPORTANT - The return codes for function tbStartStatus are different than those for function tbStart (@see tbStart).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_START_OK               (  2) - Normal execution, tbStart has completed successfully.
 * * TB_START_LOADING          (  1) - Normal execution, tbStart is still working.
 * * TB_START_NOT_CALLED       (  0) - tbStart has not been called.
 * * TB_START_CONNECTION_ERROR ( -2) - tbStart has completed unsuccessfully due to comms issue with the Trackerbird Server.
 * * TB_START_AUTH_FAILURE     ( -3) - tbStart has completed unsuccessfully due to login issue with the Trackerbird Server.
 * * TB_START_SERVER_ERROR     ( -4) - tbStart has completed unsuccessfully due to internal issue with the Trackerbird Server.
 * * TB_INVALID_PARAMETER      ( -9) - tbInstance is NULL.
 * * TB_INTERNAL_ERROR         (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbStartStatus(TBINSTANCE* tbInstance);

/**
 * @details This function is used to Track and Report an Exception that has been generated by the Trackerbird SDK Client.
 *
 * IMPORTANT - Function tbExceptionTrack does not have to be called, but if it is called, it must be after calling function
 * tbStart (@see tbStart).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param className
 * The name of the class from which the Exception occurred.  The value can be NULL or empty, but cannot contain a NEWLINE char.
 *
 * @param methodName
 * The name of the method/function from which the Exception occurred.  The value can be NULL or empty, but cannot contain a
 * NEWLINE char.
 *
 * @param exceptionMessage
 * The message contained in the Exception or text to be logged with the Exception.  The value can be NULL or empty.  The value
 * can contain multiple lines.
 *
 * @param stackTrace
 * The full or partial stack trace of the Exception to be logged with the Exception.  The value can be NULL or empty.  The value
 * can contain multiple lines.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - Exception tracking feature is disabled.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL, or className or methodName contains newline character.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbExceptionTrack(TBINSTANCE* tbInstance, const char* className, const char* methodName, const char* exceptionMessage,
                          const char* stackTrace);

/**
 * @details This function is used to Track an Event in the Trackerbird SDK Client which no Custom Value needs to be recorded.  To
 * record a Custom Value with the Event use tbEventTrackNum (@see tbEventTrackNum) or tbEventTrackTxt (@see tbEventTrackTxt).
 * 
 * An Event must have a non-empty Event Name.  If User Sessions are being used (@see tbCreateConfig and @see tbSessionStart)
 * an Event must have a recognized Session ID.  If User Sessions are not being used (@see tbCreateConfig), an Event cannot have
 * a Session ID (it must be NULL or empty).
 *
 * Optionally, Categories can be used to organize Events on the Trackerbird Reports, making the Event Data more readable.  Each
 * Event can be associated with a single Category via the supplied parameter.  If a Category Name is not desired, this parameter
 * should be NULL or empty.
 *
 * Event Names and Categories have restricted lengths.  Exceeding the maximum length results in truncation of the value.  This is
 * influenced by allowExtendedNames:
 * * If allowExtendedNames is true, Event Name and Category can each be up to 256 characters, not including the terminating
 *       NULL, with no further restriction on their combined length.
 * * If allowExtendedNames is false, Event Name and Category can each be up to 40 characters (not including the terminating
 *       NULL), and combined, they can be up to 50 characters (not including the terminating NULL).  Exceeding the combined
 *       length restriction preserves the Category (up to 40 characters), with the remaining length used by as much of the
 *       Event Name that can fit.
 *
 * IMPORANT - The Event Name and Category will be visible in the Trackerbird Analytics Reports.  Therefore, meaningful and
 * structured names are recommended to make it easier to identify Event Data in the Reports.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param category
 * The name of the Category that this Event belongs. A value of NULL or empty means no Category.  The value cannot contain a
 * NEWLINE character.  The value is length constrained and is influenced by allowExtendedNames and potentially influenced by the
 * length of eventName.
 *
 * @param eventName
 * The name of the Event to Track. The value cannot be NULL, empty nor contain a NEWLINE character.  The value is length
 * constrained and is influenced by allowExtendedNames and potentially influenced by the length of category.
 *
 * @param sessionID
 * The legal values depend on whether multiple Sessions are enabled (@see tbCreateConfig).
 * * If multiple Sessions are enabled, then the value must be a Session ID previously used in a call to function tbSessionStart
 *       (@see tbSessionStart) that has not yet been used in a call to function tbSessionStop (@see tbSessionStop).
 * * If multiple Sessions are not enabled, then the value must be NULL or empty.
 *
 * @param allowExtendedNames
 * Influences the individual and combined lengths of Event Name and Event Category.  See details above.
 * 
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.  NOTE:  If the given event has not been whitelisted in the Trackerbird
 *                                    server, then the event will not actually be tracked.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - event tracking feature can be disabled.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL; eventName NULL, empty or contains newline character; category has
 *                                    newline char; sessionID not recognized and multiSessionEnabled = true; or sessionID
 *                                    not NULL/empty and multiSessionEnabled = false.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbEventTrack(TBINSTANCE* tbInstance, const char* category, const char* eventName, const char* sessionID,
                      bool allowExtendedNames);

/**
 * @details This function is used to Track an Event in the Trackerbird SDK Client which needs to record a numeric Custom Value.
 * To record a textual Custom Value with the Event use tbEventTrackTxt (@see tbEventTrackTxt), and use tbEventTrack (@see
 * tbEventTrack) when no Custom Value is needed.  The supplied numeric Custom Value is used in the Trackerbird Event Tracking
 * Report to produce averages.  This is typically used to record the time it takes for a process to finish, or some other
 * numeric value that can be used to calculate a specific trend.
 * 
 * An Event must have a non-empty Event Name.  If User Sessions are being used (@see tbCreateConfig and @see tbSessionStart)
 * an Event must have a recognized Session ID.  If User Sessions are not being used (@see tbCreateConfig), an Event cannot have
 * a Session ID (it must be NULL or empty).
 *
 * Optionally, Categories can be used to organize Events on the Trackerbird Reports, making the Event Data more readable.  Each
 * Event can be associated with a single Category via the supplied parameter.  If a Category Name is not desired, this parameter
 * should be NULL or empty.
 *
 * Event Names and Categories have restricted lengths.  Exceeding the max length results in truncation of the value.  This is
 * influenced by allowExtendedNames:
 * * If allowExtendedNames is true, Event Name and Category can each be up to 256 characters, not including the terminating
 *       NULL, with no further restriction on their combined length.
 * * If allowExtendedNames is false, Event Name and Category can each be up to 40 characters (not including the terminating
 *       NULL), and combined, they can be up to 50 characters (not including the terminating NULL).  Exceeding the combined
 *       length restriction preserves the Category (up to 40 characters), with the remaining length used by as much of the
 *       Event Name that can fit.
 *
 * IMPORANT - The Event Name and Category will be visible in the Trackerbird Analytics Reports.  Therefore, meaningful and
 * structured names are recommended to make it easier to identify Event Data in the Reports.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param category
 * The name of the Category that this Event belongs. A value of NULL or empty means no Category.  The value cannot contain a
 * NEWLINE character.  The value is length constrained and is influenced by allowExtendedNames and potentially influenced by the
 * length of eventName.
 *
 * @param eventName
 * The name of the Event to Track. The value cannot be NULL, empty nor contain a NEWLINE character.  The value is length
 * constrained and is influenced by allowExtendedNames and potentially influenced by the length of category.
 *
 * @param customValue
 * A numeric Custom Value related to this particular event.
 *
 * @param sessionID
 * The legal values depend on whether multiple Sessions are enabled (@see tbCreateConfig).
 * * If multiple Sessions are enabled, then the value must be a Session ID previously used in a call to function tbSessionStart
 *       (@see tbSessionStart) that has not yet been used in a call to function tbSessionStop (@see tbSessionStop).
 * * If multiple Sessions are not enabled, then the value must be NULL or empty.
 *
 * @param allowExtendedNames
 * Influences the individual and combined lengths of Event Name and Event Category.  See details above.
 * 
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.  NOTE:  If the given event has not been whitelisted in the Trackerbird
 *                                    server, then the event will not actually be tracked.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - Exception tracking feature is disabled.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL; eventName NULL, empty or contains newline character; category has
 *                                    newline char; sessionID not recognized and multiSessionEnabled = true; or sessionID
 *                                    not NULL/empty and multiSessionEnabled = false.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbEventTrackNum(TBINSTANCE* tbInstance, const char* category, const char* eventName, double customValue,
                         const char* sessionID, bool allowExtendedNames);

/**
 * @details This function is used to Track an Event in the Trackerbird SDK Client which needs to record a textual Custom Value.
 * To record a numeric Custom Value with the Event use tbEventTrackNum (@see tbEventTrackNum), and use tbEventTrack (@see
 * tbEventTrack) when no Custom Value is needed.  The supplied textual Custom Value is used in the Trackerbird Event Tracking
 * Report.  This is typically used to record data collected from the Client or User, such as comments, the brand of GPU on the
 * machine, etc.
 * 
 * An Event must have a non-empty Event Name.  If User Sessions are being used (@see tbCreateConfig and @see tbSessionStart)
 * an Event must have a recognized Session ID.  If User Sessions are not being used (@see tbCreateConfig), an Event cannot have
 * a Session ID (it must be NULL or empty).
 *
 * Optionally, Categories can be used to organize Events on the Trackerbird Reports, making the Event Data more readable.  Each
 * Event can be associated with a single Category via the supplied parameter.  If a Category Name is not desired, this parameter
 * should be NULL or empty.
 *
 * Event Names and Categories have restricted lengths.  Exceeding the max length results in truncation of the value.  This is
 * influenced by allowExtendedNames:
 * * If allowExtendedNames is true, Event Name and Category can each be up to 256 characters, not including the terminating
 *       NULL, with no further restriction on their combined length.
 * * If allowExtendedNames is false, Event Name and Category can each be up to 40 characters (not including the terminating
 *       NULL), and combined, they can be up to 50 characters (not including the terminating NULL).  Exceeding the combined
 *       length restriction preserves the Category (up to 40 characters), with the remaining length used by as much of the
 *       Event Name that can fit.
 *
 * IMPORANT - The Event Name and Category will be visible in the Trackerbird Analytics Reports.  Therefore, meaningful and
 * structured names are recommended to make it easier to identify Event Data in the Reports.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param category
 * The name of the Category that this Event belongs. A value of NULL or empty means no Category.  The value cannot contain a
 * NEWLINE character.  The value is length constrained and is influenced by allowExtendedNames and potentially influenced by the
 * length of eventName.
 *
 * @param eventName
 * The name of the Event to Track. The value cannot be NULL, empty nor contain a NEWLINE character.  The value is length
 * constrained and is influenced by allowExtendedNames and potentially influenced by the length of category.
 *
 * @param customValue
 * A textual Custom Value related to this particular event. The max length of this text is 4096 characters, but this can be
 * changed via a request to Trackerbird.
 *
 * @param sessionID
 * The legal values depend on whether multiple Sessions are enabled (@see tbCreateConfig).
 * * If multiple Sessions are enabled, then the value must be a Session ID previously used in a call to function tbSessionStart
 *       (@see tbSessionStart) that has not yet been used in a call to function tbSessionStop (@see tbSessionStop).
 * * If multiple Sessions are not enabled, then the value must be NULL or empty.
 *
 * @param allowExtendedNames
 * Influences the individual and combined lengths of Event Name and Event Category.  See details above.
 * 
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.  NOTE:  If the given event has not been whitelisted in the Trackerbird
 *                                    server, then the event will not actually be tracked.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - Exception tracking feature is disabled.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL; eventName NULL, empty or contains newline character; category has
 *                                    newline char; sessionID not recognized and multiSessionEnabled = true; or sessionID
 *                                    not NULL/empty and multiSessionEnabled = false.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbEventTrackTxt(TBINSTANCE* tbInstance, const char* category, const char* eventName, const char* customValue,
                         const char* sessionID, bool allowExtendedNames);

/**
 * @details This function tests the connectivity between the Trackerbird SDK and the Trackerbird Server.  Function tbConfigCheck
 * is similar to function tbConnectionCheck (@see tbConnectionCheck) but uses the Trackerbird SDK Configuration Data (@see
 * tbCreateConfig) for the CallHome URL and Product ID values instead of function parameters.
 *
 * IMPORTANT - Function tbConfigCheck is generally not needed, as each Trackerbird SDK API requiring connectivity to the
 * Trackerbird Server does its own connectivity test.  If function tbConfigCheck is called, it must be after calling function
 * tbCreateConfig (@see tbCreateConfig).  Function tbConnectionCheck (@see tbConnectionCheck) can be called before calling
 * function tbCreateConfig.
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_CONNCHECK_MISMATCH ( -1) - Successfully reached the Trackerbird Server, but the URL and ProductID are not consistent.
 * * TB_CONN_ERROR         ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TB_AUTH_FAILURE       ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TB_SERVER_ERROR       ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TB_CONFIG_NOT_CREATED ( -5) - Configuration not successfully loaded.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbConfigCheck(TBINSTANCE* tbInstance);

/**
 * @details This function tests the connectivity between the Trackerbird SDK and the Trackerbird Server.  Function
 * tbConnectionCheck is similar to function tbConfigCheck (@see tbConfigCheck) but uses parameters for the CallHome URL and
 * Product ID rather than the Trackerbird SDK Configuration Data.
 * 
 * IMPORTANT - Function tbConnectionCheck is generally not needed, as each Trackerbird SDK API requiring connectivity to the
 * Trackerbird Server does its own connectivity test.  Function tbConnectionCheck can be called before calling function
 * tbCreateConfig (@see tbCreateConfig).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param url
 * The URL to test connectivity to (usually the Trackerbird CallHome URL).
 *
 * @param productID
 * Optionally, the Product ID for the Trackerbird Client.
 * * If NULL or empty, the Product ID is not used.
 * * Otherwise, the connectivity tests will validate that the URL is the correct one for the supplied Product ID, resulting in
 *       return code TB_CONNCHECK_MISMATCH (-1) if the validation fails.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                 (  1) - Normal execution.
 * * TB_CONNCHECK_MISMATCH ( -1) - Successfully reached the Trackerbird Server, but the URL and ProductID are not consistent.
 * * TB_CONN_ERROR         ( -2) - Unsuccessful due to comms issue with the Trackerbird Server.
 * * TB_AUTH_FAILURE       ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TB_SERVER_ERROR       ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TB_INVALID_PARAMETER  ( -9) - tbInstance is NULL.
 * * TB_INTERNAL_ERROR     (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbConnectionCheck(TBINSTANCE* tbInstance, const char* url, const char* productID);

/**
 * @details This function requests a Manual ReachOut™ Message from the Trackerbird Server. This can be either a plain text
 * Message or a URL Message, depending on what Messages are waiting on the Trackerbird Server.  To request a Manual ReachOut™
 * Message of a specific type, then use function tbMessageCheckWithMessageType (@see tbMessageCheckWithMessageType).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param message
 * This is an out parameter.  The Trackerbird SDK will always allocate memory and return it to the Trackerbird SDK Client.
 * This memory must be freed by calling function tbFree (@see tbFree) when the Trackerbird SDK Client no longer needs it.  The
 * value of the string will be one of the following:
 * * If return code >= 0, the value is the Manual ReachOut™ Message.  Max length will be 256 characters plus the NULL.
 * * Otherwise, the value is an empty string.
 *
 * @param messageType
 * This is an out parameter.  The value is set to indicate the ReachOut™ Message Type that was received.  The possible values
 * are as follows:
 * * If return code >= 0, the messageType will be TB_MESSAGETYPE_TEXT (1) or TB_MESSAGETYPE_URL (2) to describe Message Type in
 *       message.
 * * Otherwise, the messageType will be undefined.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * If =0, no message available on the Trackerbird Server.  Memory for message is allocated, and message value is empty string.
 * * If >0, the message count on the Trackerbird Server including the one returned now (so remaining messages are return value
 *       minus 1).  Memory for message is allocated, and message value is the Manual ReachOut™ Message.
 * * If <0, an error has occurred.  Memory for message is allocated, and message value is empty string.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - Message Check threshold limit has been reached.
 * * TB_CONN_ERROR            ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TB_AUTH_FAILURE          ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TB_SERVER_ERROR          ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbMessageCheck(TBINSTANCE* tbInstance, char*& message, int* messageType);

/**
 * @details This function requests a Manual ReachOut™ Message from the Trackerbird Server of a specific Message Type (either
 * a plain text Message or a URL Message).  To request a Manual ReachOut™ Message regardless of Message Type, then use function
 * tbMessageCheck (@see tbMessageCheck).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param message
 * This is an out parameter.  The Trackerbird SDK will always allocate memory and return it to the Trackerbird SDK Client.
 * This memory must be freed by calling function tbFree (@see tbFree) when the Trackerbird SDK Client no longer needs it.  The
 * value of the string will be one of the following:
 * * If return code >= 0, the value is the Manual ReachOut™ message.  Maximum length will be 256 characters plus the NULL.
 * * Otherwise, the value is an empty string.
 *
 * @param messageType
 * The Message Type to request.  The possible values are as follows:
 * * TB_MESSAGETYPE_ALL  (0) - Request any Message Type (i.e., Text or URL).
 * * TB_MESSAGETYPE_TEXT (1) - Request only Text Message Type.
 * * TB_MESSAGETYPE_URL  (2) - Request only URL Message Type.
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * If =0, no message available on the Trackerbird Server.  Memory for message is allocated, and message value is empty string.
 * * If >0, the message count on the Trackerbird Server including the one returned now (so remaining messages are return value
 *       minus 1).  Memory for message is allocated, and message value is the Manual ReachOut™ Message.
 * * If <0, an error has occurred.  Memory for message is allocated, and message value is empty string.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - Message Check threshold limit has been reached.
 * * TB_CONN_ERROR            ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TB_AUTH_FAILURE          ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TB_SERVER_ERROR          ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbMessageCheckWithMessageType(TBINSTANCE* tbInstance, char*& message, int messageTypeExpected);

/**
 * @details This function validates a license key with the Blacklist stored on the Trackerbird Server.  Five (5) License Status
 * Flags are returned for the supplied license key.  These values can be set with function tbSetLicense (@see tbSetLicense).
 * Function tbKeyCheck is a passive function that only returns data from the Trackerbird Server.  To combine a key check with a
 * key registration, use function tbKeyChanged (@see tbKeyChanged).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param licKey
 * The license key to be tested.  The value cannot be NULL or empty.
 *
 * @param tbLicenseArray
 * This is an out parameter.  The Trackerbird SDK will always allocate memory and return it to the Trackerbird SDK Client.
 * This memory must be freed by calling function tbFree (@see tbFree) when the Trackerbird SDK Client no longer needs it.  The
 * value will be a 5-element integer array containing the License Status Flag values at the following indexes:
 * * TB_KEY_ACTIVE_INDEX    (0)
 * * TB_KEY_BLACKLIST_INDEX (1)
 * * TB_KEY_EXPIRED_INDEX   (2)
 * * TB_KEY_WHITELIST_INDEX (3)
 * * TB_KEY_TYPE_INDEX      (4)
 *
 * NOTE:  Any of the array elements can be set to -1 to indicate the data is not available.
 *
 * Each of the first four (4) array elements will be set to one of the following values:
 * * TB_KEY_STATUS_UNCHANGED (-1) - Meaning Key Status is Unknown.
 * * TB_KEY_STATUS_NO        ( 0)
 * * TB_KEY_STATUS_YES       ( 1)
 *
 * The last array element (KEYTYPE) will be set to one of the following License Type values:
 * * TB_FUNCTION_NOT_AVAILABLE (-1) - Meaning License Management has been disabled, Key Type or Key is unknown, or there was a
 *                                    communications problem with the Trackerbird Server.
 * * TB_KEYTYPE_EVALUATION     ( 0)
 * * TB_KEYTYPE_PURCHASED      ( 1)
 * * TB_KEYTYPE_FREEWARE       ( 2)
 * * TB_KEYTYPE_UNKNOWN        ( 3)
 * * TB_KEYTYPE_NFR            ( 4)
 * * TB_KEYTYPE_CUSTOM1        ( 5)
 * * TB_KEYTYPE_CUSTOM2        ( 6)
 * * TB_KEYTYPE_CUSTOM3        ( 7)
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - Key Check threshold limit has been reached.
 * * TB_CONN_ERROR            ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TB_AUTH_FAILURE          ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TB_SERVER_ERROR          ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL, or licKey is NULL or empty.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbKeyCheck(TBINSTANCE* tbInstance, const char* licKey, int*& tbLicenseArray);

/**
 * @details This function validates and registers a new license key with the Trackerbird Server.  Function tbKeyChanged is
 * similar to function tbKeyCheck (@see tbKeyCheck), except rather than just being a passive license check, function
 * tbKeyChanged also registers the new key with the Trackerbird Server and associates it with this particular Trackerbird SDK
 * Client.
 *
 * Five (5) License Status Flags are returned for the supplied license key.  These values can be set with function tbSetLicense
 * (@see tbSetLicense).
 *
 * @param tbInstance
 * Pointer to the Trackerbird SDK Instance returned from function tbCreateInstance.
 *
 * @param newKey
 * The license key to be tested and changed.  The value cannot be NULL or empty.
 *
 * @param tbLicenseArray
 * This is an out parameter.  The Trackerbird SDK will always allocate memory and return it to the Trackerbird SDK Client.
 * This memory must be freed by calling function tbFree (@see tbFree) when the Trackerbird SDK Client no longer needs it.  The
 * value will be a 5-element integer array containing the License Status Flag values at the following indexes:
 * * TB_KEY_ACTIVE_INDEX    (0)
 * * TB_KEY_BLACKLIST_INDEX (1)
 * * TB_KEY_EXPIRED_INDEX   (2)
 * * TB_KEY_WHITELIST_INDEX (3)
 * * TB_KEY_TYPE_INDEX      (4)
 *
 * NOTE:  Any of the array elements can be set to -1 to indicate the data is not available.
 *
 * Each of the first four (4) array elements will be set to one of the following values:
 * * TB_KEY_STATUS_UNCHANGED (-1) - Meaning Key Status is Unknown.
 * * TB_KEY_STATUS_NO        ( 0)
 * * TB_KEY_STATUS_YES       ( 1)
 *
 * The last array element (KEYTYPE) will be set to one of the following License Type values:
 * * TB_FUNCTION_NOT_AVAILABLE (-1) - Meaning License Management has been disabled, Key Type or Key is unknown, or there was a
 *                                    communications problem with the Trackerbird Server.
 * * TB_KEYTYPE_EVALUATION     ( 0)
 * * TB_KEYTYPE_PURCHASED      ( 1)
 * * TB_KEYTYPE_FREEWARE       ( 2)
 * * TB_KEYTYPE_UNKNOWN        ( 3)
 * * TB_KEYTYPE_NFR            ( 4)
 * * TB_KEYTYPE_CUSTOM1        ( 5)
 * * TB_KEYTYPE_CUSTOM2        ( 6)
 * * TB_KEYTYPE_CUSTOM3        ( 7)
 *
 * @exception None.
 *
 * @return One of the return status constants below:
 * * TB_OK                    (  1) - Normal execution.
 * * TB_FUNCTION_NOT_AVAIL    ( -1) - Key Check threshold limit has been reached.
 * * TB_CONN_ERROR            ( -2) - Unsuccessful due to communications issue with the Trackerbird Server.
 * * TB_AUTH_FAILURE          ( -3) - Unsuccessful due to login issue with the Trackerbird Server.
 * * TB_SERVER_ERROR          ( -4) - Unsuccessful due to internal issue with the Trackerbird Server.
 * * TB_NOT_STARTED           ( -7) - Configuration not successfully started.
 * * TB_INVALID_PARAMETER     ( -9) - tbInstance is NULL, or newKey is NULL or empty.
 * * TB_APP_STOPPED           (-14) - Function not available as tbStop has already been called.
 * * TB_INTERNAL_ERROR        (-99) - Unrecoverable error detected prior to or during this call.  Make no more SDK calls.
 */
TBRESULT tbKeyChanged(TBINSTANCE* tbInstance, const char* newKey, int*& tbLicenseArray);

/**
 * @details This function frees memory that was allocated by the Trackerbird SDK and returned to the Client.  Memory allocation
 * occurs in functions tbMessageCheck (@see tbMessageCheck), tbMessageCheckWithMessageType (@see tbMessageCheckWithMessageType),
 * tbKeyCheck (@see tbKeyCheck), and tbKeyChanged (@see tbKeyChanged).
 *
 * IMPORTANT - Function tbFree should not be used for general memory management.  It should only be called for pointers to
 * memory allocated by the Trackerbird SDK.  Failing to call function tbFree will result in memory leaks.
 *
 * @param arg
 * The pointer to be freed. 
 *
 * @exception None.
 *
 * @return None.
 */
void tbFree(void* arg);

#ifdef __cplusplus
} // extern "C"
#endif

#endif // TRACKERBIRD_H
